@@ -1,45 +1,42 @@
 import dataclasses
 
+import collections.abc
 import datetime
 import decimal
+import enum
 import functools
 import json
-import typing as t
+import typing
 import uuid
-from enum import Enum
-from starlette.datastructures import FormData
 
 JSONEncoder = json.JSONEncoder
 
+_type_to_encoder: dict[type, typing.Callable] = {
+    uuid.UUID: str,
+    datetime.timedelta: str,
+    datetime.datetime: lambda x: x.isoformat(),
+    datetime.date: lambda x: x.isoformat(),
+    datetime.time: lambda x: x.isoformat(),
+    set: list,
+    frozenset: list,
+    collections.abc.KeysView: list,
+    collections.abc.ValuesView: list,
+    decimal.Decimal: str,
+    enum.Enum: lambda x: x.value,
+}
 
-def json_default(o: t.Any) -> t.Any:
+
+def json_default(o: typing.Any) -> typing.Any:
     """Usage: json.dumps(data, default=json_default)"""
     if hasattr(o, "to_json"):
         return o.to_json()
 
-    if isinstance(o, (uuid.UUID, datetime.timedelta)):
-        return str(o)
+    if hasattr(o, "__json__"):
+        return o.__json__()
 
-    if isinstance(o, datetime.datetime):
-        return o.isoformat()
-
-    if isinstance(o, datetime.date):
-        return o.isoformat()
-
-    if isinstance(o, datetime.time):
-        return o.isoformat()
-
-    if isinstance(o, (set, frozenset)):
-        return list(o)
-
-    if isinstance(o, decimal.Decimal):
-        return str(o)
-
-    if isinstance(o, Enum):
-        return o.value
-
-    if isinstance(o, FormData):
-        return dict(o)
+    for type_class, encoder in _type_to_encoder.items():
+        if isinstance(o, type_class):
+            return encoder(o)
 
     if dataclasses.is_dataclass(o):
         return dataclasses.asdict(o)
@@ -47,7 +44,7 @@ def json_default(o: t.Any) -> t.Any:
     return JSONEncoder().default(o)
 
 
-def walk(value: t.Any, callback: t.Callable) -> t.Any:
+def walk(value: typing.Any, callback: typing.Callable) -> typing.Any:
     """Visit each item and apply"""
     if isinstance(value, dict):
         for key, value_ in value.items():
@@ -66,32 +63,28 @@ def walk(value: t.Any, callback: t.Callable) -> t.Any:
         return value
 
 
-def jsonify(value: t.Any) -> t.Any:
+def jsonify(value: typing.Any) -> typing.Any:
     """Visit each item and convert it into JSON serializable value."""
     return walk(value, functools.partial(json_default))
 
 
-JSONData = t.Any  # https://github.com/python/typing/issues/182
+JSONData = typing.Any  # https://github.com/python/typing/issues/182
 
 
 def dump(
     value: JSONData,
-    fp: t.IO[str],
-    default: t.Callable[[t.Any], t.Any] = json_default,
-    **kwargs: t.Any,
+    fp: typing.IO[str],
+    default: typing.Callable[[typing.Any], typing.Any] = json_default,
+    **kwargs: typing.Any,
 ) -> None:
     json.dump(value, fp, default=default, **kwargs)
 
 
-def dumps(value: JSONData, **kwargs: t.Any) -> str:
+def dumps(value: JSONData, **kwargs: typing.Any) -> str:
     if "default" not in kwargs and "cls" not in kwargs:
         kwargs["default"] = json_default
     return json.dumps(value, **kwargs)
 
 
-def load(fp: t.IO[str], **kwargs: t.Any) -> str:
-    return json.load(fp, **kwargs)
-
-
-def loads(value: str, **kwargs: t.Any) -> JSONData:
-    return json.loads(value, **kwargs)
+load = json.load
+loads = json.loads
